package cn.zhangdeman.context;

import cn.zhangdeman.consts.RecordField;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.Getter;
import lombok.Setter;

import java.io.Serializable;
import java.util.Map;

// 初始化的请求信息
@Getter
@Setter
@JsonIgnoreProperties(ignoreUnknown = true) // 忽略未知属性字段
public class RuntimeContext implements Serializable {
    @JsonProperty(RecordField.REQUEST_ID)
    private String requestId; // 本次请求request id
    @JsonProperty(RecordField.SERVER_IP)
    private String serverIp; // 服务器Ip
    @JsonProperty(RecordField.SERVER_HOSTNAME)
    private String serverHostname; // 服务器hostname
    @JsonProperty(RecordField.TRACE_ID)
    private String traceId; // 全局trace id
    @JsonProperty(RecordField.START_TIMESTAMP)
    private Long startTimeStamp; // 开始请求时间
    @JsonProperty(RecordField.REQUEST_INFO)
    private Request requestInfo; // 请求信息
    @JsonIgnore
    private Response<Object> response; // 响应信息
    @JsonIgnore
    private HttpServletRequest httpServletRequest;// 请求实例
    @JsonIgnore
    private HttpServletResponse httpServletResponse; // 相应实例

    // 序列化
    @Override
    public String toString() {
        ObjectMapper mapper = new ObjectMapper();
        try {
            return mapper.writeValueAsString(this);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

    // 序列化
    public String serializable() {
        return toString();
    }

    // 反序列化
    public RuntimeContext unserializable(String json) throws JsonProcessingException {
        ObjectMapper mapper = new ObjectMapper();
        return mapper.readValue(json, new TypeReference<RuntimeContext>() {
        });
    }
}
